from AccessControl import ClassSecurityInfo, Permissions
from Products.CMFCore import CMFCatalogAware
from Products.CMFCore import CMFCorePermissions
from Products.Archetypes.public import BaseContent, registerType
from Products.Archetypes.public import BaseSchema, Schema
from Products.Archetypes.public import TextField, StringField, \
    DateTimeField, LinesField, IntegerField, BooleanField, ComputedField
from Products.Archetypes.public import KeywordWidget, StringWidget, \
    CalendarWidget, ComputedWidget, BooleanWidget
from Products.TimeTracker.config import PROJECTNAME, NULL_DATE
from DateTime import DateTime
import operator

schema = BaseSchema.copy() + Schema((
    BooleanField('Running',
        index = 'FieldIndex',
        widget=BooleanWidget(
            label='Running',
            label_msgid='label_running',
            i18n_domain='timetracker',
            visible=0
        ),
    ),
    BooleanField('Pending',
        index = 'FieldIndex',
        widget=BooleanWidget(
            label='Pending',
            label_msgid='label_pending',
            i18n_domain='timetracker',
            visible=0
        ),
    ),
    BooleanField('Sticky',
        index = 'FieldIndex',
        widget=BooleanWidget(
            label='Sticky',
            label_msgid='label_sticky',
            i18n_domain='timetracker',
            visible=0
        ),
    ),
    StringField('Activity',
        required= 1,
        index = 'TextIndex',
        searchable = 1,
        widget=StringWidget(
            label='Activity',
            label_msgid='label_activity',
            i18n_domain='timetracker'
        ),
    ),
    StringField('Category',
        required = 1,
        index='TextIndex',
        searchable = 1,
        widget=StringWidget(
            label='Category',
            label_msgid='label_category',
            description='The category associated with this task',
            description_msgid='help_category',
            i18n_domain='timetracker'
        ),
    ),
    DateTimeField('StartDate',
        index='FieldIndex',
        widget=CalendarWidget(
            label='Start date',
            label_msgid='label_start_date',
            i18n_domain='timetracker'
        ),
    ),
    DateTimeField('EndDate',
        index='FieldIndex',
        widget=CalendarWidget(
            label='End date',
            label_msgid='label_end_date',
            i18n_domain='timetracker'
        ),
    ),
    ComputedField('Duration',
        expression = 'here.getDuration()',
        widget=ComputedWidget(
            label='Duration',
            label_msgid='label_duration',
            description='The time (in secs) spent on this task',
            description_msgid='help_duration',
            i18n_domain='timetracker',
            visible=0
        ),
    ),
    ComputedField('TitleFieldIndex',
        expression = 'here.Title()',
        index = 'FieldIndex',
        widget=ComputedWidget(
            visible=0
        ),
    ),
    ))
short_name = schema['id'] 
short_name.widget.visible = False

class Task(BaseContent):
    security = ClassSecurityInfo()
    archetype_name = 'Task'
    schema = schema
    immediate_view = 'base_edit'
    default_view ='base_view'
    allowed_content_types = ()
    global_allow = 0 
    filter_content_types = 0
    use_folder_tabs = 0
    _at_rename_after_creation = True

    __implements__ = BaseContent.__implements__
        
    actions = (
        { 'id':'activities',
                'name':'Activities',
                'action':'string:${object_url}/task_activities',
                'permissions':(CMFCorePermissions.View,),
        },
    )
    
    security.declarePublic('displayContentsTab')
    def displayContentsTab(self):
        """ Always hide the contents tab
        """
        return False

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'stopTask')
    def stopTask(self, REQUEST=None):
        self.setRunning(False)
        self.setEndDate(DateTime())
        self.setSticky(False)
        self.setPending(False)
        self.reindexObject()
        if REQUEST:
            url = REQUEST.URL1 + '/timetracker_tasks'
            REQUEST.RESPONSE.redirect(url)
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'getDuration')
    def getDuration(self, REQUEST=None):
        """ Compute the amount of time spent on a task """              
        secs = 0
        if not self.getPending():
            if self.getEndDate() == DateTime(NULL_DATE):
                secs = DateTime().timeTime()- self.getStartDate().timeTime()
            if not (self.getEndDate() == DateTime(NULL_DATE)):
                secs = self.getEndDate().timeTime()-self.getStartDate().timeTime()
        return secs

    security.declareProtected(Permissions.view, 'getStartTime')
    def getStartTime(self, REQUEST=None):
        """ Return only time that the task was started """              
        if self.getStartDate():
            return DateTime.TimeMinutes(DateTime(self.getStartDate() or None))

    security.declareProtected(Permissions.view, 'getEndTime')
    def getEndTime(self, REQUEST=None):
        """ Return only time that the task has ended """              
        if self.getEndDate():
            return DateTime.TimeMinutes(DateTime(self.getEndDate()))

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setEndDate')
    def setEndDate(self, date):
        """ Override the default mutator """              
        endDate = self.Schema()['EndDate']
        endDate.set(self, date) 
        if date != NULL_DATE:
            self.setRunning(False)
       
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setEndDate')
    def setStartDate(self, date):
        """ 
        Override the default mutator 
        XXX End_date is not set when startDate is set in future
        """              
        startDate = self.Schema()['StartDate']
        startDate.set(self, date) 
        if DateTime(date).isFuture():
            self.setPending(True)
            self.setRunning(False)
            self.setSticky(False)

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'getActivities')
    def getActivities(self, REQUEST=None):
        """ Find all the tasks with the same Title field """              
        search_string = self.Title()
        parent = self.aq_parent
        proxies = self.portal_catalog(getTitleFieldIndex=search_string,
            path='/'.join(parent.getPhysicalPath()))
        tasks = [p.getObject() for p in proxies]
        deprecated = [(t.getEndDate(), t) for t in tasks]
        deprecated.sort()
        tasks = [b for a,b in deprecated]
        return tasks
    
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'startTask')
    def startTask(self, startDate):
        """ 
        Initialise the fields in a task's schema
        """
        self.setPending(False)
        self.setRunning(True)
        self.setSticky(False)
        self.setStartDate(startDate)
        self.setEndDate(NULL_DATE)
        self.reindexObject()

    security.declareProtected(Permissions.view, 'timeSpent')
    def timeSpent(self):
        """ 
        Return the amount of hours and minutes spent on a currently running task 
        """
        if not self.getPending():
            time = int(self.getDuration()/60)
            hours = int(time/60)
            minutes = int(time%60)
        else:
            hours = 0
            minutes = 0
        return hours, minutes   

    security.declareProtected(Permissions.view, 'totalTime')
    def totalTime(self):
        """ 
        Return the total amount of time (including previous activities) 
        associated with the currently running task
        """
        search_string= self.Title()
        time = 0
        proxies =self.portal_catalog(getTitleFieldIndex = search_string,getPending=False,
            path='/'.join(self.aq_parent.getPhysicalPath()))
        for p in proxies:
            task = p.getObject()
            time = time + task.getDuration()
        time = int(operator.div(time,60))
        hours = int(operator.div(time,60))
        minutes = int(time%60)
        return hours,minutes
                    
registerType(Task, PROJECTNAME)

def modify_fti(fti):
    d = {}
    for a in list(fti['actions']) + list(Task.actions):
        d[a['id']] = a
    # we need the 'view' action to keep Plone happy, we just make it invisible
    d['view']['visible'] = False
    alist = []
    # append actions in right order
    for action_id in ('edit', 'activities'):
        alist.append(d[action_id])
    fti['actions'] = tuple(alist)
    return fti
