""" 
Worflow for Task objects in the TimeTracker
"""
from Products.CMFCore.WorkflowTool import addWorkflowFactory
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from Products.DCWorkflow.Transitions import TRIGGER_AUTOMATIC
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent

def setupTimeTrackerTaskWorkflow(wf):
    for s in ('pending', 'billed'):             # add wf states
        wf.states.addState(s)
    wf.transitions.addTransition('bill')       # add wf transition
    wf.variables.setStateVar('review_state')   
    wf.permissions+=(ModifyPortalContent,)
    wf.states.setInitialState('pending')

    # Set state properties
    sdef = wf.states['pending']                 
    sdef.setProperties(                
        title='Pending',
        transitions=('bill',))
    sdef.setPermission(ModifyPortalContent, 0, ('Manager','Member',))

    sdef = wf.states['billed']            
    sdef.setProperties(                 
        title='Billed',
    )
    sdef.setPermission(ModifyPortalContent, 0, ('Manager',))

    # Set transition properties
    tdef = wf.transitions['bill']
    tdef.setProperties(
        title='Bill this task',
        new_state_id='billed',
        trigger_type=1,                 #Trigger Type = initiated by user action
        actbox_name='Bill',
        actbox_url='',
        props={'guard_expr':"python:here.taskAllowed2Bill()"}
    )
 
def createTimeTrackerTaskWorkflow(id):
    ob=DCWorkflowDefinition(id)
    setupTimeTrackerTaskWorkflow(ob)
    ob.setProperties(title='TimeTracker Task Workflow')
    return ob

addWorkflowFactory(createTimeTrackerTaskWorkflow,
    id='timetracker_task_workflow',title='TimeTracker Task Workflow')
